﻿//{$WARNINGS OFF}
//{$WARN SYMBOL_PLATFORM OFF}
//{$WARN UNIT_PLATFORM OFF}
//=============================================================================
//  ブラウザに URL をセットしてそのページを表示
//  キー操作をエミュレート
//
//-----------------------------------------------------------------------------
//
//  【履歴】
//
//  2021年11月13日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows [Version 10.0.19042] Pro 64 bit + Delphi XE5(UP2) Pro VCL-32
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
 private
    { Private 宣言 }
  public
    { Public 宣言 }
  end;

  function IsImeOnWnd(AWnd: HWND): Boolean;
  procedure ImeOnOffWnd(AWnd: HWND; IsOn: Boolean);

var
  Form1: TForm1;

implementation

uses
  Winapi.Imm,
  plSendShortCut;

{$R *.dfm}

const
  IMC_GETOPENSTATUS = $05;
  IMC_SETOPENSTATUS = $06;

//=============================================================================
//  ウェブブラウザのアドレスバーに URL セットしてその URL のページを表示
//  キー操作をエミュレート
//
//  動作確認方法
//  (1) ウェブブラウザを起動
//  (2) このプログラムの TButton をクリック
//
//  plSendShortCut (ショートカットキー送信操作用ユニット) の SendInputText 使用
//  SendInputText 手続きは Windows API の SendInput 関数を使用している
//=============================================================================
procedure TForm1.Button1Click(Sender: TObject);
var
  LTargetWnd : HWND;
begin
  // このフォームを表示する前にアクティブだったウィンドウのハンドルを取得
  LTargetWnd := GetNextWindow(Handle, GW_HWNDNEXT);
  while not IsWindowVisible(LTargetWnd) do begin
    LTargetWnd := GetNextWindow(LTargetWnd, GW_HWNDNEXT);
  end;


  SendShortCutKeyEx(LTargetWnd, 'Alt + D');
  SendShortCutKeyEx(LTargetWnd, 'Delete');

  // 日本語 IME を OFF にする
  ImeOnOffWnd(LTargetWnd, False);

  SendInputText(LTargetWnd, 'http://mrxray.on.coocan.jp/index.htm');
  SendShortCutKeyEx(LTargetWnd, 'Enter');
end;

//=============================================================================
//  ウェブブラウザのアドレスバーに URL セットしてその URL のページを表示
//
//  SendInputText で送信した文字列は IME が ON だと未確定の文字列になる
//  その場合は [Enter] キーを送信して確定する
//=============================================================================
procedure TForm1.Button2Click(Sender: TObject);
var
  LTargetWnd : HWND;
begin
  // このフォームを表示する前にアクティブだったウィンドウのハンドルを取得
  LTargetWnd := GetNextWindow(Handle, GW_HWNDNEXT);
  while not IsWindowVisible(LTargetWnd) do begin
    LTargetWnd := GetNextWindow(LTargetWnd, GW_HWNDNEXT);
  end;


  SendShortCutKeyEx(LTargetWnd, 'Alt + D');
  SendShortCutKeyEx(LTargetWnd, 'Delete');

  SendInputText(LTargetWnd, 'http://mrxray.on.coocan.jp/index.htm');
  // 日本語 IME が ON だったら変換を確定する
  if IsImeOnWnd(LTargetWnd) then begin
    SendShortCutKeyEx(LTargetWnd, 'Enter');
  end;
  SendShortCutKeyEx(LTargetWnd, 'Enter');
end;

//-----------------------------------------------------------------------------
//  他のアプリの日本語 IME の ON / OFF を調べる
//  この関数は，自アプリにおける検出は不可
//  uses に Winapi.Imm が必要
//-----------------------------------------------------------------------------
function IsImeOnWnd(AWnd: HWND): Boolean;
var
  LhIMC : HIMC;
begin
  LhImc  := ImmGetDefaultIMEWnd(AWnd);
  Result := Boolean(SendMessage(LhImc, WM_IME_CONTROL, IMC_GETOPENSTATUS, 0));
end;
//-----------------------------------------------------------------------------
//  他のアプリの日本語 IME の ON / OFF を切り換える
//  この手続きは，自アプリに対する操作は不可
//  uses に Winapi.Imm が必要
//-----------------------------------------------------------------------------
procedure ImeOnOffWnd(AWnd: HWND; IsOn: Boolean);
var
  LhIMC : HIMC;
begin
  LhImc := ImmGetDefaultIMEWnd(AWnd);
  SendMessage(LhImc, WM_IME_CONTROL, IMC_SETOPENSTATUS, LPARAM(IsOn));
end;

end.
