﻿//{$WARNINGS OFF}
//{$WARN SYMBOL_PLATFORM OFF}
//{$WARN UNIT_PLATFORM OFF}
//=============================================================================
//  ウェブブラウザで表示中のページのソースコードを取得
//  ウェブブラウザのショートカットキー操作を使用する例
//
//-----------------------------------------------------------------------------
//
//  【履歴】
//
//  2021年11月13日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows [Version 10.0.19042] Pro 64 bit + Delphi XE5(UP2) Pro VCL-32
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  NkClipDetector;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    NkClipDetector1: TNkClipDetector;
    procedure Button1Click(Sender: TObject);
    procedure NkClipDetector1ClipChange(Sender: TObject);
 private
    { Private 宣言 }
  public
    { Public 宣言 }
  end;

  function QueryFullProcessImageNameW(Process: THandle; Flags: DWORD; Buffer: PChar;
    Size: PDWORD): DWORD; stdcall; external 'kernel32.dll';
  function EnumTopWindowExeName(hWindow: HWND; lPar: LPARAM): BOOL; stdcall;

var
  Form1: TForm1;

implementation

uses
  Vcl.Clipbrd,
  Winapi.Dwmapi,
  System.StrUtils,
  plSendShortCut;

{$R *.dfm}

var
  FTargetWnd : HWND;

//=============================================================================
//  ウェブブラウザで表示中のページのソースコードを取得
//  ウェブブラウザのショートカットキー操作を使用する例
//
//  FireFox, IE 11 では動作しない
//  FireFox は [Ctrl] + [A] の操作でブラウザがハングアップ
//  全て選択状態にして [Ctrl] + [C] を送信してもハングアップする
//  IE 11 は管理者への昇格ダイアログが表示され，別ウィンドウ表示
//=============================================================================
procedure TForm1.Button1Click(Sender: TObject);
var
  LExeName : string;
  LArgName : string;
begin
  Clipboard.Clear;
  Memo1.Lines.Clear;
  Self.FormStyle := fsStayOnTop;


  // 対象のブラウザの EXE 名
  // LExeName := 'chrome.exe';
  LExeName := 'MSEdge.exe';

  // 起動しているかをチェック
  LArgName := LExeName;
  EnumWindows(@EnumTopWindowExeName, LPARAM(@LArgName));

  FTargetWnd := StrToIntDef(LArgName, 0);
  if FTargetWnd = 0 then Exit;


  // ソースコード表示と全選択・コピーの処理はタイムラグがある 
  SendShortCutKeyEx(FTargetWnd, 'Ctrl + U');
  Sleep(100);
  SendShortCutKeyEx('Ctrl + A');
  SendShortCutKeyEx('Ctrl + C');
end;

//=============================================================================
//  TNkClipDetector の OnClipChange イベント
//  TNkClipDetector は中村拓男氏作のクリップボード監視用コンポーネント
//
//  ソースコード取得操作前に Clipboard.Clear を実行しておく
//=============================================================================
procedure TForm1.NkClipDetector1ClipChange(Sender: TObject);
begin
  if Clipboard.HasFormat(CF_TEXT) then begin
    SendShortCutKeyEx(FTargetWnd, 'Ctrl + F4');
    Sleep(50);
    Memo1.Lines.Text := Clipboard.AsText;
    SetForegroundWindow(Handle);
    Self.FormStyle := fsNormal;
  end;
end;

//-----------------------------------------------------------------------------
//  EnumWindows のコールバック関数
//  引数 (lPar) と同じ EXE 名の EXE が起動しているかを検出
//  検出したらウィンドウハンドルの値を文字列にして引数 lPar に返す
//
//  Microsoft Edge の EXE 名は Chromium 版と非 Chromium 版とでは異なる
//   非 Chromium 版では microsoftedge.exe
//   Chromium 版では MSEdge.exe
//-----------------------------------------------------------------------------
function EnumTopWindowExeName(hWindow: HWND; lPar: LPARAM): BOOL; stdcall;
const
  PROCESS_QUERY_LIMITED_INFORMATION = $1000;
  DWMWA_CLOAKED = 14;
var
  LExeName    : string;
  LCloaked    : BOOL;
  LProcessID  : DWORD;
  LhProcess   : THandle;
  LBuffer     : array [0..MAX_PATH - 1] of Char;
  LSTR_SIZE   : DWORD;
  LAppPath    : string;
begin
  Result := True;
  LExeName := LowerCase(PString(lPar)^);

  if GetWindow(hWindow, GW_OWNER) <> 0 then exit;
  if not IsWindowVisible(hWindow) then exit;

  // 待機中またはバックグラウンドのウィンドウは対象外
  DwmGetWindowAttribute(hWindow, DWMWA_CLOAKED, @LCloaked, SizeOf(LCloaked));
  if LCloaked then Exit;


  // プロセス ID を取得．その値からプロセスのハンドル取得
  GetWindowThreadProcessId(hWindow, Addr(LProcessID));
  LhProcess := OpenProcess(PROCESS_QUERY_LIMITED_INFORMATION, False, LProcessID);
  try
    // プロセスのハンドルから EXE のフルパスを取得
    FillChar(LBuffer, SizeOf(LBuffer), #0);
    LSTR_SIZE := Length(LBuffer);
    if QueryFullProcessImageNameW(LhProcess,
                                  0,
                                  @LBuffer[0],
                                  @LSTR_SIZE) = 0 then Exit;
    LAppPath := LBuffer;
    LAppPath := ExtractFileName(LAppPath);

    if RightStr(LExeName, 4) <> '.exe' then LExeName := LExeName + '.exe';
    if SameText(LAppPath, LExeName) then begin
      PString(lPar)^ := IntToStr(hWindow);
      Result := False;
    end;
  finally
    CloseHandle(LhProcess);
  end;
end;

end.
